require 'spec_helper'

class BaseSpec < Spec

  let(:service_pack)  { Class.new(Base) }
  let(:services)      { service_pack.new }

  it 'provides predicate for checking registration' do
    service_pack.register :test, Class.new

    assert_equal true,  service_pack.registered?(:test)
    assert_equal false, service_pack.registered?(:other)
  end

  it 'does not affect other derived classes' do
    other_service_pack = Class.new(Base)

    service_pack.register :test, Class.new

    assert_equal false, other_service_pack.registered?(:test)
    refute_respond_to other_service_pack.new, :test
  end

  it 'passes the base instance to the instanciated service' do
    service_pack.register :foo, Struct.new(:base)

    assert_equal services, services.foo.base
  end

  it 'raises an error if service is not registered' do
    exception = assert_raises NotImplementedError do
      services[:some_service]
    end
  end

  it 'allows overwriting of a registered service' do
    service_class       = Struct.new(:arg)
    other_service_class = Struct.new(:arg)

    service_pack.register :abc, service_class
    service_pack.register :abc, other_service_class

    assert_kind_of other_service_class, services.abc
  end


  describe 'service access' do
    let(:service_class) { Struct.new(:arg) }
    before do
      service_pack.register :abc, service_class
    end

    describe 'access to a service instance' do
      it 'is possible via method' do
        assert_kind_of service_class, services.abc
      end

      it 'is possible via index operator' do
        assert_kind_of service_class, services[:abc]
      end
    end

    it 'caches the service instance' do
      assert_equal services.abc.object_id, services[:abc].object_id
    end
  end
end
